//
//  AppDelegate.m
//  MobileAcceleratorExample
//
//  Created by Brian Salomon on 9/16/15.
//  Copyright © 2015 Akamai. All rights reserved.
//

#import <VocSdk/VocSdk.h>

#import "AppDelegate.h"

@interface AppDelegate () <VocServiceDelegate>

@end

@implementation AppDelegate


- (BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions
{
	[[UIApplication sharedApplication] registerForRemoteNotifications];

    NSError *error = nil;
    self.akaService = [VocServiceFactory createAkaWebAcceleratorWithDelegate:self
															   delegateQueue:[NSOperationQueue mainQueue]
																	 options:nil
																	   error:&error];
    if (!self.akaService) {
        // service cannot start
        NSLog(@"Could not create service %@", error);
        return NO;
    }

    if (self.akaService.state == VOCServiceStateNotRegistered) {
        // service needs registering
        NSLog(@"SDK user not registered, starting registration flow");
        return YES;
    }

    // service already registered
    NSLog(@"SDK user registered, starting normal flow");

    // example one-time event log
    [self.akaService logEvent:@"APP_LAUNCHED"];

    // example timed event log
	NSString *eventName = @"Sample Event";
    [self.akaService startEvent:eventName];
	// perform actions to time between startEvent and stopEvent
	[self.akaService stopEvent:eventName];

    return YES;
}

- (void)applicationWillResignActive:(UIApplication *)application {
}

- (void)applicationDidEnterBackground:(UIApplication *)application {
}

- (void)applicationWillEnterForeground:(UIApplication *)application {
}

- (void)applicationDidBecomeActive:(UIApplication *)application {
}

- (void)applicationWillTerminate:(UIApplication *)application {
}


#pragma mark - VocServiceDelegate

- (void)vocService:(id<VocService>)vocService didBecomeNotRegistered:(NSDictionary *)info
{
    NSLog(@"didBecomeNotRegistered  %@ %@", vocService, info);
	[[NSNotificationCenter defaultCenter] postNotificationName:registrationUpdate object:nil];
}

- (void)vocService:(id<VocService>)vocService didFailToRegister:(NSError *)error
{
    NSLog(@"didFailToRegister  %@ %@", vocService, error);
}

- (void)vocService:(id<VocService>)vocService didRegister:(NSDictionary *)info
{
    NSLog(@"didRegister %@ %@", vocService, info);
	[[NSNotificationCenter defaultCenter] postNotificationName:registrationUpdate object:nil];
}

- (void)vocService:(nonnull id<VocService>)vocService didInitialize:(nonnull NSDictionary *)info
{
    NSLog(@"didInitialize %@ %@", vocService, info);
}


#pragma mark Push Notifications

- (void)application:(UIApplication*)application didRegisterForRemoteNotificationsWithDeviceToken:(NSData*)deviceToken
{
	[self.akaService setDevicePushToken:deviceToken];
}

- (void)application:(UIApplication*)application didFailToRegisterForRemoteNotificationsWithError:(NSError*)error
{
	NSLog(@"Failed to get push token. Error: %@", error);
}

- (void)application:(UIApplication *)application didReceiveRemoteNotification:(NSDictionary *)userInfo fetchCompletionHandler:(void (^)(UIBackgroundFetchResult result))completionHandler
{
	if ([self.akaService application:application didReceiveRemoteNotification:userInfo fetchCompletionHandler:completionHandler]) {
		// remote notification was for akaService
		return;
	}
	// otherwise handle remote notification intended for host app
	completionHandler(UIBackgroundFetchResultNoData);
}

- (void)application:(UIApplication *)application handleActionWithIdentifier:(NSString *)identifier forRemoteNotification:(NSDictionary *)userInfo completionHandler:(void(^)())completionHandler
{
	NSLog(@"Remote notification with identifier: %@, payload: %@", identifier, userInfo);

	[self application:application didReceiveRemoteNotification:userInfo fetchCompletionHandler:completionHandler];
}


@end
