//
//  AppDelegate.swift
//  MobileAcceleratorExampleSwift
//
//  Created by Waazim Reza on 23/05/2016.
//  Copyright © 2016 Akamai. All rights reserved.
//

import UIKit

//Identifier for registartion update notification
let registrationUpdate = "registrationUpdate"

@UIApplicationMain

class AppDelegate: UIResponder, UIApplicationDelegate , VocServiceDelegate {

	var window: UIWindow?
	var akaService: AkaWebAccelerator?

	func application(_ application: UIApplication, didFinishLaunchingWithOptions launchOptions: [UIApplication.LaunchOptionsKey : Any]? = nil) -> Bool {

		do {
			try akaService = VocServiceFactory.createAkaWebAccelerator(with: self, delegateQueue: OperationQueue.main, options: nil)
		} catch {
			print("MAP service could not be created")
		}

		// example one-time event log
		akaService?.logEvent("APP_LAUNCHED")

		// example timed event log
		let eventName: String = "Sample Event"
		akaService?.startEvent(eventName)
		// ... perform actions over time ...
		akaService?.stopEvent(eventName)

		if (akaService?.state == VOCServiceState.notRegistered) {
			print("MAP not registered, starting registration flow")
		} else {
			print("MAP already registered")
		}

		return true
	}

	//MARK: MAP SDK optional notifiers

	func vocService(_ vocService: VocService, didRegister info: [AnyHashable : Any]) {
		// update menu screen
		NotificationCenter.default.post(name: Notification.Name(registrationUpdate), object: nil)
		print("MAP registered: \(vocService), \(info)")
	}

	func vocService(_ vocService: VocService, didBecomeNotRegistered info: [AnyHashable : Any]) {
		// update menu screen
		NotificationCenter.default.post(name: Notification.Name(registrationUpdate), object: nil)
		print("MAP became unregistered: \(vocService), \(info)")
	}

	func vocService(_ vocService: VocService, didFailToRegister error: Error) {
		print("MAP register failure: \(vocService), \(error)")
	}

	func vocService(_ vocService: VocService, didInitialize info: [AnyHashable : Any]) {
		print("MAP initialized: \(vocService), \(info)")
	}
}
